"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FastABI = void 0;
const bignumber_js_1 = require("bignumber.js");
var uuid = require('uuid');
var addon = require('../native');
function isObject(arg) {
    return arg.constructor.name === 'Object';
}
function isString(arg) {
    return typeof arg === 'string' || arg instanceof String;
}
class FastABI {
    constructor(abi, opts) {
        this._opts = Object.assign({ BigNumber: bignumber_js_1.BigNumber }, opts) || { BigNumber: bignumber_js_1.BigNumber };
        this._key = uuid.v4();
        this._abi = abi;
        addon.loadAbi(this._key, JSON.stringify(abi));
    }
    static ping() {
        addon.hello();
    }
    encodeInput(fnName, values) {
        const found = this._abi.filter((a) => a.name === fnName);
        const args = this._convertArgs(values, found[0]);
        try {
            const encoded = addon.encodeInput(this._key, fnName, args);
            return `0x${encoded}`;
        }
        catch (e) {
            throw new Error(`${e.message}.\nvalues=${JSON.stringify(values)}\nargs=${JSON.stringify(args)}`);
        }
    }
    decodeInput(fnName, output) {
        const found = this._abi.filter((a) => a.name === fnName)[0];
        const decoded = addon.decodeInput(this._key, fnName, output);
        return this._convertDataItems(found.inputs, decoded);
    }
    decodeOutput(fnName, output) {
        const found = this._abi.filter((a) => a.name === fnName)[0];
        const decoded = addon.decodeOutput(this._key, fnName, output);
        return this._convertDataItems(found.outputs, decoded);
    }
    _convertDataItem(abi, value) {
        if (abi.type.indexOf('[]') !== -1) {
            return value.map((v) => this._convertDataItem(Object.assign(Object.assign({}, abi), { type: abi.type.split('[]')[0] }), v));
        }
        if (abi.type.indexOf('int') !== -1) {
            return new this._opts.BigNumber(value);
        }
        if (abi.type === 'tuple' && abi.components) {
            const output = {};
            for (const [i, c] of Object.entries(abi.components)) {
                output[c.name] = this._convertDataItem(c, value[parseInt(i)]);
            }
            return output;
        }
        return value;
    }
    _convertDataItems(abi, values) {
        const output = [];
        for (const [i, v] of Object.entries(abi)) {
            output.push(this._convertDataItem(v, values[parseInt(i)]));
        }
        if (abi.length === 1) {
            return output[0];
        }
        return output;
    }
    _convertArg(arg) {
        if (arg === undefined) {
            throw new Error(`Encountered undefined argument`);
        }
        // Remove any prepending 0x
        if (typeof arg === 'string' || arg instanceof String) {
            arg;
        }
        if (Array.isArray(arg)) {
            return arg.map((a) => this._convertArg(a));
        }
        if (arg.constructor.name === 'Object') {
            return Object.values(arg).map((a) => this._convertArg(a));
        }
        return arg.toString();
    }
    _convertArgs(args, abi) {
        return args.map((a, i) => {
            if (isObject(a)) {
                // Ensure the values are in the correct order
                const clone = {};
                abi.inputs[i].components.map((k) => (clone[k.name] = a[k.name]));
                return this._convertArg(clone);
            }
            return this._convertArg(a);
        });
    }
}
exports.FastABI = FastABI;
//# sourceMappingURL=fast_abi.js.map